{=============================================================================}
{ HLTOOLS                                                                     }
{=============================================================================}
{ Additional routines to make your Delphi-life easier,....                    }
{=============================================================================}
{ VERSION 1.0; Created GetVolumeLabel, GetDirTree and BytoToStr for files     }
{ VERSION 1.1; Added THLRegIniFile for working with the registry              }
{ VERSION 1.2; Added Load- and SaveWindowPosition for saving form dimensions  }
{              in the registry (HKEY_CURRENT_USER)                            }
{ VERSION 1.3; Added BoolToStr en StrToBool conversion routines               }
{ VERSION 1.4; Added LoadRegString, LoadRegFloat, LoadRegInteger, LoadRegBool,}
{              SaveRegString, SaveRegFloat, SaveRegInteger, SaverRegBool      }
{              for easy storing data to the registery (HKEY_CURRENT_USER)     }
{=============================================================================}

unit HLTools;

interface

uses Windows, Classes, SysUtils, Registry, Forms;

{ REGISTRY TYPE }
type
   THLRegIniFile = class(TRegIniFile)
   public
      constructor Create(const FileName: string; AKey: HKEY);
   end;

{ FILESYSTEM FUNCTIONS }
function BytesToStr(Nr:Cardinal) : string;                         { Convert bytes to a shorter format, ie. Kb/Mb }
function GetVolumeLabel(DriveChar: Char): string;                  { Read VOLUME-LABEL of a drive                 }
procedure GetDirTree(Path:string; TheList:TStrings);               { Read the tree of a 'path', ie. 'C:\WINDOWS'  }
{ CONVERSION FUNCTIONS }
function BoolToStr(ValueTrue : boolean) : string;                  { convert booleanvalue to string - True/False }
function StrToBool(ValueTrue : string) : Boolean;                  { convert stringvalue to boolean - True/False }
{ REGISTRY FUNCTIONS }
procedure SaveRegBool(MyApp, MyKey : string; MyValue : boolean);   { Save a boolean value to registry }
function  LoadRegBool(MyApp, MyKey : string):boolean;             { Load a boolean value from registry }
procedure SaveRegFloat(MyApp, MyKey : string; MyValue : double);   { Save a float/double value to registry }
function  LoadRegFloat(MyApp, MyKey : string):double;              { Load a float/double value from registry }
procedure SaveRegInteger(MyApp, MyKey : string; MyValue : integer);{ Save a integer value to registry }
function  LoadRegInteger(MyApp, MyKey : string):integer;           { Load a integer value from registry }
procedure SaveRegString(MyApp, MyKey, MyValue : string);           { Save a string value to registry }
function  LoadRegString(MyApp, MyKey : string):string;             { Load a string value from registry }

procedure SaveWindowPosition(Sender : TForm; MyApp : string);      { Load & Save Form positions }
procedure LoadWindowPosition(Sender : TForm; MyApp : string);      { from/into the registry }

implementation

{=============================================================================}
{ BOOLTOSTR                                                                   }
{=============================================================================}
function BoolToStr(ValueTrue : boolean) : string;
{=============================================================================}
{ Convert a boolean value to a string, ie. True or False                      }
{=============================================================================}
begin
  if ValueTrue then
    result:='True'
  else
    result:='false';
end;

{=============================================================================}
{ BOOLTOSTR                                                                   }
{=============================================================================}
function StrToBool(ValueTrue : string) : Boolean;
{=============================================================================}
{ Convert a string value to a boolean, ie. True or False                      }
{=============================================================================}
begin
  if uppercase(ValueTrue)='TRUE' then
    result:=True
  else
    result:=false;
end;

{=============================================================================}
{ LOADREGBOOL and SAVEREGBOOL                                                 }
{=============================================================================}
procedure SaveRegBool(MyApp, MyKey : string; MyValue : boolean);
{=============================================================================}
{ The function SaveRegXXXXX (where XXXXX in [ Bool, Float, Integer, String ]) }
{ Loads a value to the registry, example:                                     }
{                                                                             }
{ SaveRegString('MyApplication','MyKey','MyValue');                           }
{                                                                             }
{ Similar goes for the LoadRegXXXXX:                                          }
{                                                                             }
{ MyValue:=LoadRegString('MyApplication','MyKey');                            }
{                                                                             }
{ the returned value is of type XXXXX !! Default value is FALSE, 0 or ''.     }
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 MyReg.WriteBool('',MyKey,MyValue);
 MyReg.Free;
end;
{=============================================================================}
function LoadRegBool(MyApp, MyKey : string):boolean;
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 result:=MyReg.ReadBool('',MyKey,false);
 MyReg.Free;
end;

{=============================================================================}
{ SAVEREGFLOAT and LOADREGFLOAT                                               }
{=============================================================================}
procedure SaveRegFloat(MyApp, MyKey : string; MyValue : double);
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 MyReg.WriteString('',MyKey,FloatToStr(MyValue));
 MyReg.Free;
end;
{=============================================================================}
function LoadRegFloat(MyApp, MyKey : string):double;
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 result:=StrToFloat(MyReg.ReadString('',MyKey,'0'));
 MyReg.Free;
end;

{=============================================================================}
{ LOADREGINTEGER and SAVEREGINTEGER                                           }
{=============================================================================}
procedure SaveRegInteger(MyApp, MyKey : string; MyValue : integer);
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 MyReg.WriteInteger('',MyKey,MyValue);
 MyReg.Free;
end;
{=============================================================================}
function LoadRegInteger(MyApp, MyKey : string):integer;
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 result:=MyReg.ReadInteger('',MyKey,0);
 MyReg.Free;
end;

{=============================================================================}
{ LOADREGSTRING and SAVEREGSTRING                                             }
{=============================================================================}
procedure SaveRegString(MyApp, MyKey, MyValue : string);
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 MyReg.WriteString('',MyKey,MyValue);
 MyReg.Free;
end;
{=============================================================================}
function LoadRegString(MyApp, MyKey : string):string;
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 result:=MyReg.ReadString('',MyKey,'');
 MyReg.Free;
end;

{=============================================================================}
{ LOADWINDOWPOSITION and SAVEWINDOWPOSITION                                   }
{=============================================================================}
procedure SaveWindowPosition(Sender : TForm; MyApp : string);
{=============================================================================}
{ in FORM.ONCREATE -> LoadWindowPosition(self,'MyWonderfullApplication');     }
{ This will save LEFT, TOP, WIDTH and HEIGHT of the form in the registery     }
{ like this:                                                                  }
{                                                                             }
{ HKEY_CURRENT_USER\Software\MyWonderfullAppication\MyForm_Left               }
{ HKEY_CURRENT_USER\Software\MyWonderfullAppication\MyForm_Top                }
{ HKEY_CURRENT_USER\Software\MyWonderfullAppication\MyForm_Width              }
{ HKEY_CURRENT_USER\Software\MyWonderfullAppication\MyForm_Height             }
{                                                                             }
{ Notes:                                                                      }
{ * The screen position is stored on a USER level ! (profiles etc.)           }
{ * Make sure the FORM.POSITION:=poDESIGNED !!!!!!!!!                         }
{                                                                             }
{ usage:  LoadWindowPosition(self,'MyWonderfullApplication');                 }
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);
 MyReg.WriteInteger('',Sender.Name+'_Left',Sender.left);
 MyReg.WriteInteger('',Sender.Name+'_Top',Sender.top);
 MyReg.WriteInteger('',Sender.Name+'_Width',Sender.width);
 MyReg.WriteInteger('',Sender.Name+'_Height',Sender.height);
 MyReg.Free;
end;

procedure LoadWindowPosition(Sender : TForm; MyApp : string);
{=============================================================================}
{ in FORM.ONDESTROY -> SaveWindowPosition(self,'MyWonderfullApplication');    }
{ This one will restore the form's position when saved with the previous one  }
{ if the form position is not found the form will be screen-centered          }
{ See LoadWindowPosition for more details.                                    }
{                                                                             }
{ usage:  SaveWindowPosition(self,'MyWonderfullApplication');                 }
{=============================================================================}
var MyReg : THLRegIniFile;
    Appname : string;
    Value : integer;
begin
 Appname:='\SOFTWARE\'+MyApp;
 MyReg:=THLRegIniFile.Create(Appname,HKEY_CURRENT_USER);

 Value:=MyReg.ReadInteger('',Sender.Name+'_Left',0);
 if Value=0 then
   Value:=(Screen.width-Sender.width) div 2;
 Sender.left:=value;

 Value:=MyReg.ReadInteger('',Sender.Name+'_Top',0);
 if Value=0 then
   Value:=(Screen.height-Sender.Height) div 2;
 Sender.top:=value;

 Value:=MyReg.ReadInteger('',Sender.Name+'_Width',0);
 if Value<>0 then Sender.width:=value;

 Value:=MyReg.ReadInteger('',Sender.Name+'_Height',0);
 if Value<>0 then Sender.height:=value;

 MyReg.Free;
end;

{=============================================================================}
{ HLRegIniFile (OBJECT!)                                                      }
{=============================================================================}
constructor THLRegIniFile.Create(const FileName: string; AKey: HKEY);
{=============================================================================}
{ HLRegIniFile                                                                }
{                                                                             }
{ Small tool for easier acces to TRegIniFile                                  }
{                                                                             }
{ Example how to use:                                                         }
{                                                                             }
{ ...                                                                         }
{ var MyReg : THLRegIniFile;                                                  }
{     What  : string;                                                         }
{ ...                                                                         }
{ MyReg:=THLRegIniFile.Create('\SOFTWARE\Classes',HKEY_LOCAL_MACHINE);        }
{ What:=MyReg.ReadString('Drive\DefaultIcon','','Error: Did not find it!');   }
{ MyReg.Free;                                                                 }
{ ...                                                                         }
{                                                                             }
{ In 'What' youcan find (for example) 'C:\WINDOWS\SYSTEM\COOL.DLL,0'          }
{ or 'Error: Did not find it!'.                                               }
{=============================================================================}
begin
   RootKey := AKey;
   LazyWrite := True;
   OpenKey(FileName, True);
end;

{==============================================================================}
{ String:=ConvertMemory(Number);                                               }
{==============================================================================}
function BytesToStr(Nr:Cardinal) : string;
{==============================================================================}
{ ConvertMemory convert a cardinal number to a string, using bytes, Kb or Mb   }
{ formatted text.                                                              }
{ ie:  12345678 bytes will become 11 Mb                                        }
{==============================================================================}
begin
  if Nr > 1048575 then
    Result:=inttostr(Nr div 1048575)+' Mb'
  else if Nr > 1024 then
    Result:=inttostr(Nr div 1024)+' Kb'
  else
    Result:=IntToStr(Nr)+' bytes';
end;
{==============================================================================}

{==============================================================================}
{ string:=GetVolumeLabel(DriveLetter);                                         }
{==============================================================================}
function GetVolumeLabel(DriveChar: Char): string;
{==============================================================================}
{ GetVolumeLabel, return the label of the media, currently in/at DriveChar     }
{ i.e.  Label1.Caption:=GetVolumeLabel('C');                                   }
{ returns the volume label of drive C.                                         }
{ Original code: Borland, FILECTL.PAS                                          }
{==============================================================================}
var
  OldErrorMode: Integer;                { Store old Error-mode }
  NotUsed, VolFlags: Integer;           { Dummies }
  Buf: array [0..MAX_PATH] of Char;     { Create charater buffer for labelname }
begin
  OldErrorMode:=SetErrorMode(SEM_FAILCRITICALERRORS);   { Avoid silly errors }

  try
    GetVolumeInformation(PChar(DriveChar + ':\'), Buf, sizeof(Buf),
      nil, NotUsed, VolFlags, nil, 0);     { Try to get Volume-Info }
    SetString(Result, Buf, StrLen(Buf));   { Set return result }
    if DriveChar < 'a' then                { Adapt Charcase to the case of DriveChar }
      Result:=AnsiUpperCase(Result)
    else
      Result:=AnsiLowerCase(Result);
  finally
    SetErrorMode(OldErrorMode);            { Restore errormode }
  end;
end;
{==============================================================================}

{==============================================================================}
{ RecurseDirs(Path,StringList);                                                }
{==============================================================================}
procedure GetDirTree(Path:string; TheList:TStrings);
{==============================================================================}
{ Starts reading the directory, starting at 'path', recursing through all (sub)}
{ directories. Directories are tagged with a <DIR>                             }
{ ie:                                                                          }
{ ...                                                                          }
{   ListBox1 : TListBox;                                                       }
{ ...                                                                          }
{   GetDirTree('C:\WINDOWS\',ListBox1.Items);                                  }
{==============================================================================}
var SearchRec : TSearchRec; { for storing file-info   }
    ErrorCode : Integer;    { catching the resultcode }
begin
  ErrorCode:=FindFirst(Path+'*',faAnyFile,SearchRec); { Locate first file in path }
  { Note: the PATH must include the "\"-symbol, ie: 'C:\' }
  while ErrorCode=0 do { File-information succesfully read }
    begin
      if SearchRec.Name[1]<>'.' then { avoid '.' and '..' directories }
        begin
          if (SearchRec.Attr AND faDirectory)>0 then { is this a directory ? }
            begin
              TheList.Add(Path+SearchRec.Name+'<DIR>'); { Add Directory to list }
              GetDirTree(Path+SearchRec.Name+'\',TheList); { Read this directory first }
            end { if Directory }
          else
            TheList.Add(Path+SearchRec.Name); { Add file to the list }

        end; { if SearchRec }
        ErrorCode:=FindNext(SearchRec);  { Read next fat-entry }
      end; { while ErrorCode }

  FindClose(SearchRec);
end;
{==============================================================================}

end.
